const sqlRegexes = [
  new RegExp(/^select /),
  new RegExp(/^update /),
  new RegExp(/^insert into /),
  new RegExp(/^delete from /),
  new RegExp(/^create table /),
  new RegExp(/^create index /),
  new RegExp(/^create db /),
  new RegExp(/^alter table /),
  new RegExp(/^alter index /),
  new RegExp(/^alter db /),
  new RegExp(/^drop table /),
  new RegExp(/^alter index /),
  new RegExp(/^alter db /),
  new RegExp(/^truncate table /)
]

function isSql(query) {
  if (!query) {
    return false;
  }

  query = query.trim().toLowerCase();
  for (let i = 0; i < sqlRegexes.length; ++i) {
    if (sqlRegexes[i].test(query)) {
      return true;
    }
  }

  return false;
}

function validate(node, context) {
  if (!node) {
    return;
  }

  if (node.type === "Literal" && typeof node.value === "string" && isSql(node.value)) {
    context.report(node, "Possible SQL query, use the 'SQL' tagged template for raw SQL queries instead of a string");
  }

  if (node.type === "TaggedTemplateExpression" && node.tag.name !== "SQL") {
    node = node.quasi;
  }

  if (node.type === "TemplateLiteral" && node.expressions.length) {
    const literal = node.quasis.map(quasi => quasi.value.raw).join("_");
    if (isSql(literal)) {
      context.report(node, "Use the 'SQL' template tag for raw SQL queries");
    }
  }
}

module.exports.rules = {
  "no-unsafe-sql": context => ({
    CallExpression(node) {
      node.arguments.forEach(argument => validate(argument, context));
    },
    VariableDeclaration(node) {
      node.declarations.forEach(declaration =>
        validate(declaration.init, context)
      );
    }
  })
};
